package org.netkernel.mod.architecture.bits;

import java.util.concurrent.atomic.AtomicInteger;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.meta.IEndpointStateMeta;
import org.netkernel.layer0.meta.IModuleMeta;
import org.netkernel.layer0.meta.IRepresentationMeta;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.mod.architecture.MenuAccessor;
import org.netkernel.mod.architecture.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;

public class AllModules extends StandardAccessorImpl
{
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String type=aContext.getThisRequest().getArgumentValue("type");
		if (type.equals("allModules"))
		{	onAllModules(aContext);
		}
		else if (type.equals("allRepresentations"))
		{	onAllRepresentations(aContext);
		}
		else if (type.equals("allTransports"))
		{	onAllTransports(aContext);
		}
		else if (type.equals("allProblems"))
		{	onAllProblems(aContext);
		}
	}
	
	public void onAllModules(INKFRequestContext aContext) throws Exception
	{
		String tableId="allmod-tab"+Utils.getUniqueId();
		HDSBuilder b=new HDSBuilder();
		b.pushNode("div");
		MenuAccessor.addExplorerTemplate(b,aContext,null,"All Modules",null);
		StringBuilder script=new StringBuilder();
		script.append("$(document).ready(function(){");
		script.append("initDataTable('"+tableId+"'); ");
		script.append("initLinksRaw('#"+tableId+" .el-module','gMenuModule'); ");
		script.append("});");
		b.pushNode("script",script.toString());
		b.addNode("@type","text/javascript");
		b.popNode();
		
		b.pushNode("table");
		b.addNode("@id",tableId);
		b.addNode("@style","width:100%");
		b.pushNode("thead");
		b.pushNode("tr");
		b.addNode("th","Name");
		b.addNode("th","Identifier");
		b.addNode("th","Version");
		b.addNode("th","Level");
		b.addNode("th","Errors");
		b.popNode();
		b.popNode();
		
		IHDSNode list=aContext.source("active:moduleListDoc", IHDSNode.class);
		for (IHDSNode module: list.getNodes("/modules/module"))
		{
			b.pushNode("tr");
			boolean problems=module.getFirstNode("problem")!=null;
			if (problems)
			{	b.addNode("@class", "el el-module error");
			}
			else
			{	b.addNode("@class", "el el-module");
			}
			b.addNode("@entity",module.getFirstValue("hash"));
			b.pushNode("td");
			b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-module"); b.popNode();
			b.addNode("span", module.getFirstValue("name"));
			b.popNode();
			b.addNode("td",module.getFirstValue("id"));
			b.addNode("td",module.getFirstValue("version"));
			b.addNode("td",module.getFirstValue("runlevel"));
			b.pushNode("td");
			b.pushNode("span",problems);
			b.addNode("@style","display: none;");
			b.popNode();
			b.pushNode("input");
			b.addNode("@type","checkbox");
			b.addNode("@disabled","true");
			if (problems) b.addNode("@checked","");
			b.popNode();
			b.popNode();
			b.popNode();
		}
		
		INKFResponse resp=aContext.createResponseFrom(b.getRoot());
		resp.setHeader("WrappedControlPanel", Boolean.TRUE);
	}

	public void onAllRepresentations(INKFRequestContext aContext) throws Exception
	{
		String tableId="allmod-tab"+Utils.getUniqueId();
		HDSBuilder b=new HDSBuilder();
		b.pushNode("div");
		MenuAccessor.addExplorerTemplate(b,aContext,null,"All Representations",null);
		StringBuilder script=new StringBuilder();
		script.append("$(document).ready(function(){");
		script.append("initDataTable('"+tableId+"', ");
		script.append("{ \"bAutoWidth\": false, \"aoColumnDefs\": [ { \"sWidth\": \"33%\", \"aTargets\": [ 0,1,2 ] } ] } ");
		script.append("); ");
		script.append("initLinksRaw('#"+tableId+" .el-rep','gMenuRep'); ");
		script.append("initLinksRaw('#"+tableId+" .el-module','gMenuModule'); ");
		script.append("});");
		b.pushNode("script",script.toString());
		b.addNode("@type","text/javascript");
		b.popNode();
		
		b.pushNode("table");
		b.addNode("@id",tableId);
		b.addNode("@style","width:100%");
		b.pushNode("thead");
		b.pushNode("tr");
		b.addNode("th","Name");
		b.addNode("th","Module");
		b.addNode("th","Java Class");
		b.popNode();
		b.popNode();
		
		IRepDeployedModules modules=aContext.source("active:moduleList", IRepDeployedModules.class);
		IHDSNode list=modules.getHierarchy();
		for (IHDSNode module: list.getNodes("/modules/module"))
		{	IModuleMeta moduleMeta=(IModuleMeta)modules.metadataForHash((String)module.getFirstValue("hash"));
			for (IHDSNode space: module.getNodes("spaces//space"))
			{	String spaceHash=(String)space.getFirstValue("hash");
				ISpaceMeta spaceMeta=(ISpaceMeta)modules.metadataForHash(spaceHash);
				ISpaceElements se=spaceMeta.getElements();
				for (int i=0; i<se.size(); i++)
				{	if (se.getDelegatedSpace(i)!=null) continue;
					IIdentifier eid=se.getIdentifier(i);
					if (eid==null) continue;
					String eHash=spaceHash+"/"+eid.toString();
					IMetaRepresentation eMeta=modules.metadataForHash(eHash);
					if (eMeta instanceof IRepresentationMeta)
					{	IRepresentationMeta rMeta=(IRepresentationMeta)eMeta;
						b.pushNode("tr");
						b.pushNode("td");
						b.addNode("@class","el el-rep"); b.addNode("@entity",eHash);
						b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-rep"); b.popNode();
						b.addNode("span", eMeta.getName());
						b.popNode();
						b.pushNode("td");
						b.addNode("@class","el el-module"); b.addNode("@entity",module.getFirstValue("hash"));
						b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-module"); b.popNode();
						b.addNode("span", moduleMeta.getName()+" (v"+moduleMeta.getVersion()+")");
						b.popNode();
						b.addNode("td",rMeta.getImplementation().getName());
						b.popNode();
					}
				}
			}
		}
		
		INKFResponse resp=aContext.createResponseFrom(b.getRoot());
		resp.setHeader("WrappedControlPanel", Boolean.TRUE);
	}
	
	public void onAllTransports(INKFRequestContext aContext) throws Exception
	{
		String tableId="allmod-tab"+Utils.getUniqueId();
		
		HDSBuilder b=new HDSBuilder();
		b.pushNode("div");
		MenuAccessor.addExplorerTemplate(b,aContext,null,"All Transports",null);
		StringBuilder script=new StringBuilder();
		script.append("$(document).ready(function(){");
		script.append("initDataTable('"+tableId+"', ");
		script.append("{ \"bAutoWidth\": false, \"aoColumnDefs\": [ { \"sWidth\": \"33%\", \"aTargets\": [ 0,1 ] } ] } ");
		script.append("); ");
		script.append("initLinksRaw('#"+tableId+" .el-physical','gMenuPhysical'); ");
		script.append("initLinksRaw('#"+tableId+" .el-module','gMenuModule'); ");
		script.append("});");
		b.pushNode("script",script.toString());
		b.addNode("@type","text/javascript");
		b.popNode();
		
		b.pushNode("table");
		b.addNode("@id",tableId);
		b.addNode("@style","width:100%");
		b.pushNode("thead");
		b.pushNode("tr");
		b.addNode("th","Name");
		b.addNode("th","Module");
		b.popNode();
		b.popNode();
		
		IRepDeployedModules modules=aContext.source("active:moduleList", IRepDeployedModules.class);
		IHDSNode list=modules.getHierarchy();
		for (IHDSNode module: list.getNodes("/modules/module"))
		{	IModuleMeta moduleMeta=(IModuleMeta)modules.metadataForHash((String)module.getFirstValue("hash"));
			for (IHDSNode space: module.getNodes("spaces//space"))
			{	String spaceHash=(String)space.getFirstValue("hash");
				ISpaceMeta spaceMeta=(ISpaceMeta)modules.metadataForHash(spaceHash);
				ISpaceElements se=spaceMeta.getElements();
				IEndpoint lastEp=null;
				for (int i=0; i<se.size(); i++)
				{	
					IEndpoint ep=se.getPhysicalEndpoint(i);
					if (ep!=lastEp)
					{	lastEp=ep;
						String epHash=modules.hashForComponent(ep);
						IMetaRepresentation epMeta=modules.metadataForHash(epHash);
						Object tpi=epMeta.getAdditionalFields().getValue(StandardTransportImpl.STATE_TRANSPORT_INDICATOR);
						if (tpi!=null && tpi.equals(Boolean.TRUE))
						{
							b.pushNode("tr");
							b.pushNode("td");
							b.addNode("@class","el el-physical"); b.addNode("@entity",epHash);
							b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-physical"); b.popNode();
							b.addNode("span", epMeta.getName());
							b.popNode();
							b.pushNode("td");
							b.addNode("@class","el el-module"); b.addNode("@entity",module.getFirstValue("hash"));
							b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-module"); b.popNode();
							b.addNode("span", moduleMeta.getName()+" (v"+moduleMeta.getVersion()+")");
							b.popNode();
							b.popNode();
						}
					}
				}
			}
		}
		
		INKFResponse resp=aContext.createResponseFrom(b.getRoot());
		resp.setHeader("WrappedControlPanel", Boolean.TRUE);
	}
	
	public void onAllProblems(INKFRequestContext aContext) throws Exception
	{
		String tableId="allmod-tab"+Utils.getUniqueId();
		
		HDSBuilder b=new HDSBuilder();
		b.pushNode("div");
		MenuAccessor.addExplorerTemplate(b,aContext,null,"All Problems",null);
		StringBuilder script=new StringBuilder();
		script.append("$(document).ready(function(){");
		script.append("initDataTable('"+tableId+"', ");
		script.append("{ \"bAutoWidth\": false, \"aoColumnDefs\": [ { \"sWidth\": \"33%\", \"aTargets\": [ 0,1 ] } ] } ");
		script.append("); ");
		script.append("initLinks('#"+tableId+"', '#"+tableId+" .el-physical','gMenuPhysical'); ");
		script.append("initLinks('#"+tableId+"', '#"+tableId+" .el-module','gMenuModule'); ");
		script.append("});");
		b.pushNode("script",script.toString());
		b.addNode("@type","text/javascript");
		b.popNode();
		
		b.pushNode("table");
		b.addNode("@id",tableId);
		b.addNode("@style","width:100%");
		b.pushNode("thead");
		b.pushNode("tr");
		b.addNode("th","Entity");
		b.addNode("th","Problem");
		b.addNode("th","Module");
		b.popNode();
		b.popNode();
		
		IRepDeployedModules modules=aContext.source("active:moduleList", IRepDeployedModules.class);
		IHDSNode list=modules.getHierarchy();
		for (IHDSNode module: list.getNodes("/modules/module"))
		{	IModuleMeta moduleMeta=(IModuleMeta)modules.metadataForHash((String)module.getFirstValue("hash"));
			boolean foundCause=false;
			for (IHDSNode space: module.getNodes("spaces//space"))
			{	String spaceHash=(String)space.getFirstValue("hash");
				ISpaceMeta spaceMeta=(ISpaceMeta)modules.metadataForHash(spaceHash);
				ISpaceElements se=spaceMeta.getElements();
				IEndpoint lastEp=null;
				for (int i=0; i<se.size(); i++)
				{	
					IEndpoint ep=se.getPhysicalEndpoint(i);
					if (ep!=lastEp)
					{	lastEp=ep;
						String epHash=modules.hashForComponent(ep);
						IMetaRepresentation epMeta=modules.metadataForHash(epHash);
						Object tpi=epMeta.getAdditionalFields().getValue(IEndpointStateMeta.KEY_PROBLEM);
						if (tpi!=null)
						{	foundCause=true;
							writeProblem(b, epHash, epMeta.getName(), tpi, module.getFirstValue("hash"), moduleMeta);
						}
					}
				}
			}
			Object tpi=moduleMeta.getAdditionalFields().getValue(IEndpointStateMeta.KEY_PROBLEM);
			if (tpi!=null && !foundCause)
			{	writeProblem(b, null, null, tpi, module.getFirstValue("hash"), moduleMeta);
			}
		}
		
		INKFResponse resp=aContext.createResponseFrom(b.getRoot());
		resp.setHeader("WrappedControlPanel", Boolean.TRUE);
	}
	
	private void writeProblem(HDSBuilder b, String epHash, String epName, Object tpi, Object moduleHash, IModuleMeta moduleMeta)
	{
		b.pushNode("tr");
		b.addNode("@class", "error");
		b.pushNode("td");
		if (epHash!=null)
		{	b.addNode("@class","el el-physical"); b.addNode("@entity",epHash);
			b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-physical"); b.popNode();
			b.addNode("span", epName);
		}
		b.popNode();
		
		b.addNode("td",tpi);
		
		b.pushNode("td");
		b.addNode("@class","el el-module"); b.addNode("@entity",moduleHash);
		b.pushNode("span"," "); b.addNode("@class", "ui-icon-ae ui-icon-ae-module"); b.popNode();
		b.addNode("span", moduleMeta.getName()+" (v"+moduleMeta.getVersion()+")");
		b.popNode();
		b.popNode();
	}
	
}
