<?xml version="1.0" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="xml"/>
  <xsl:param name="param"/>
  <xsl:param name="config"/>
  
  <xsl:variable name="explorer">/tools/explore/</xsl:variable>

  <xsl:include href="res:/nkse/style/xsl/format_resolution.xsl"/>	


  <xsl:template match="/" xmlns:xrl="http://netkernel.org/xrl" >
    <div>
	  <script type="text/javascript"><xsl:comment>
	  function doDisplay()
	  {	document.form1.action="/tools/requesttrace";
	  		document.form1.action1.value='display';
			document.form1.submit();
	  }
	  function doSubmit(name)
	  {	document.form1.action="";
	  		document.form1.action1.value=name;
	  		document.form1.submit();
	  }
	  </xsl:comment></script>
	  
		<xrl:include>
			<xrl:identifier>active:controlPanelInfoBox</xrl:identifier>
			<xrl:argument name="content">
				<div>
          <p>
            The Request Resolution Trace tool will create and inject a resource request
            into a specified space with a specified resource identifier and request verb.
          </p>
          <p>
            To configure a request trace:
          </p>
          <ul>
            <li>Select a module (This will populated the Space drop down list)</li>
            <li>Select a space.</li>
            <li>Enter the resource identifier.</li>
            <li>Select a request verb.</li>
          </ul>
          <p>
            To issue the request trace click on the button labeled:
          </p>
          <ul>
            <li>"Resolve" to determine if the request resolves to an endpoint.</li>
            <li>"Inject" to resolve and evaluate the request.</li>
            <li>"Display Representation" to resolve and evaluate the request and display the returned representation in the browser.</li>
          </ul>
				</div>
			</xrl:argument>
		</xrl:include>
	  
	  
	  
      <form name="form1" method="get">
        <table class="title-table" border="0" width="100%">
        
          <!-- Module Selection Row -->
          <tr>
            <td class="label">Module:</td>
            <td>
              <select name="module" onchange="doSubmit('')">
                <xsl:for-each select="/modules/module">
                  <xsl:sort select="id"/>
                  <option>
                    <xsl:attribute name="value">
                      <xsl:value-of select="hash"/>
                    </xsl:attribute>
                    <xsl:if test="$param/resolve/module/text()=hash">
                      <xsl:attribute name="selected">true</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="id"/> - <xsl:value-of select="version"/>
                  </option>
                </xsl:for-each>
              </select>
            </td>
          </tr>

          <tr>
            <td class="label">Space:</td>
            <xsl:variable name="module">
            	<xsl:choose>
            		<xsl:when test="$param/resolve/module/text()"><xsl:value-of select="$param/resolve/module/text()"/></xsl:when>
            		<xsl:otherwise>
            			<xsl:for-each select="/modules/module">
                  			<xsl:sort select="id"/>
                  			<xsl:if test="position()=1"><xsl:value-of select="hash"/></xsl:if>
                  		</xsl:for-each>
            		</xsl:otherwise>
            	</xsl:choose>
            </xsl:variable>
            <td>
              <select name="space">
                <xsl:for-each select="/modules/module[hash=$module]/spaces/space">
                	<xsl:call-template name="space-option"/>
                </xsl:for-each>
              </select>
            </td>
          </tr>

          <tr>
            <td class="label">Identifier:</td>
            <td>
              <input type="text" name="uri" style="width:40em;">
                <xsl:attribute name="value">
                  <xsl:value-of select="$param/resolve/uri"/>
                </xsl:attribute>
              </input>
            </td>
          </tr>
          <tr>
            <td class="label">Verb:</td>
            <td>
            	<select name="verb">
            		<option value="1">
	                    <xsl:if test="$param/resolve/verb/text()='1'"><xsl:attribute name="selected">true</xsl:attribute></xsl:if>
            			SOURCE
            		</option>
            		<option value="2">
	                    <xsl:if test="$param/resolve/verb/text()='2'"><xsl:attribute name="selected">true</xsl:attribute></xsl:if>
            			SINK
            		</option>
            		<option value="4">
	                    <xsl:if test="$param/resolve/verb/text()='4'"><xsl:attribute name="selected">true</xsl:attribute></xsl:if>
            			EXISTS
            		</option>
            		<option value="8">
	                    <xsl:if test="$param/resolve/verb/text()='8'"><xsl:attribute name="selected">true</xsl:attribute></xsl:if>
            			DELETE
            		</option>
            		<option value="16">
	                    <xsl:if test="$param/resolve/verb/text()='16'"><xsl:attribute name="selected">true</xsl:attribute></xsl:if>
            			NEW
            		</option>
            		<option value="256">
	                    <xsl:if test="$param/resolve/verb/text()='256'"><xsl:attribute name="selected">true</xsl:attribute></xsl:if>
            			META
            		</option>
            	</select>
            </td>
          </tr>

          <tr>
            <td/>
            <td>
              <input type="hidden" name="action1"/>
              <input class="btn" value="Resolve" type="button" onclick="doSubmit('resolve')"/>
              <input class="btn" value="Inject" type="button" onclick="doSubmit('execute')"/>
              <input class="btn" name="display" value="Display Representation" type="button" onclick="doDisplay()" />
            </td>
           </tr>

          <xsl:apply-templates select="$param/resolve/resolution"/>
          <xsl:apply-templates select="$param/resolve/response"/>
       
          
        </table>
      </form>
    </div>
  </xsl:template>
  
  <xsl:template match="resolution">
      <tr>
         <td colspan="2">
           <hr/>
         </td>
       </tr>
       <tr>
         <td class="label">Physical Endpoint:</td>
         <td class="pseudo-input">
           <div>
           	<a href="{$explorer}{physicalHash}">
           		<xsl:value-of select="toString"/>
           	</a>
           	</div>
         </td>
       </tr>

       <tr>
         <td class="label">Logical Endpoint:</td>
         <td class="pseudo-input">
           <div>
           	<a href="{$explorer}{logicalHash}">
           		<xsl:value-of select="id"/>
           	</a>
           </div>
         </td>
       </tr>
       <tr>
         <td class="label">Resolved Scope:</td>
         <td class="pseudo-input">
         	<xsl:for-each select="scope/level">
         		<div>
         			<a href="{$explorer}{hash}">
         				<xsl:value-of select="space"/>
         			</a>
         		</div>
         	</xsl:for-each>
         </td>
       </tr>
       <tr>
         <td class="label">Resolution Trace:</td>
         <td class="pseudo-input">
         	<xsl:for-each select="trace">
         		<xsl:call-template name="resolution"/>
         	</xsl:for-each>
         </td>
       </tr>
  </xsl:template>
  
	<xsl:template name="space-option">
		<xsl:param name="indent"></xsl:param>
		<option>
			<xsl:attribute name="value">
				<xsl:value-of select="hash"/>
			</xsl:attribute>
			<xsl:if test="$param/resolve/space/text()=hash">
				<xsl:attribute name="selected">true</xsl:attribute>
			</xsl:if>
			<xsl:value-of select="$indent"/><xsl:value-of select="name"/>
		</option>
		<xsl:for-each select="spaces/space">
			<xsl:call-template name="space-option">
				<xsl:with-param name="indent"><xsl:value-of select="$indent"/>&#160;&#160;</xsl:with-param>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template> 
	
	
	<xsl:template match="response">
      <tr>
         <td colspan="6">
           <hr/>
         </td>
       </tr>
		<tr>
			<td class="label">Representation Type:</td>
			<td class="pseudo-input">
				<xsl:choose>
					<xsl:when test="repHash">
						<a href="{$explorer}{repHash}">
							<xsl:value-of select="repName"/>
						</a> (<xsl:value-of select="repClass"/>)
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="repClass"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
		<tr>
			<td class="label">Representation toString():</td>
			<td colspan="5" style="border: solid 1px black; background: white;">
				<pre><xsl:value-of select="repToString"/></pre>
			</td>
		</tr>
		<tr>
			<td class="label">Expiry Determinants:</td>
			<td class="pseudo-input">
				<xsl:for-each select="expiry/e">
					<xsl:call-template name="expiry"/>
				</xsl:for-each>
			</td>
		</tr>
		<tr>
			<td class="label">IsExpired?:</td>
			<td class="pseudo-input">
				<xsl:value-of select="isExpired"/>
			</td>
		</tr>
		<tr>
			<td class="label">Cost:</td>
			<td class="pseudo-input">
				<xsl:value-of select="cost"/>
			</td>
		</tr>
		
		<tr>
			<td class="label">Response Fields:</td>
			<td class="pseudo-input">
				<xsl:choose>
					<xsl:when test="usermeta/value">
						<xsl:for-each select="usermeta/value">
							<xsl:variable name="class">
								<xsl:choose>
									<xsl:when test="position() mod 2 = 0">
										<xsl:text>evenrow</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:text>oddrow</xsl:text>
									</xsl:otherwise>  
								</xsl:choose>
							</xsl:variable>							
							<div class="{$class}">
								<xsl:choose>
									<xsl:when test="@key='ad'">
										[Additional dependencies on locally cached responses]
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="key"/> = <xsl:value-of select="text()"/>
									</xsl:otherwise>
								</xsl:choose>
								
							</div>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>none</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template name="expiry">
		<div>
			
			<xsl:value-of select="@exp"/>
			<xsl:value-of select="@msg"/>
			<xsl:if test="e">
				<div style="margin-left: 1em;">
				<xsl:for-each select="e">
					<xsl:call-template name="expiry"/>
				</xsl:for-each>
				</div>
			</xsl:if>
		</div>
	</xsl:template>		
	

</xsl:stylesheet>
